package model

import (
	"fmt"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
)

type EventCowSameTime struct {
	Id            int64                       `json:"id"`
	CowId         int64                       `json:"cowId"`
	PenId         int32                       `json:"penId"`
	PenName       string                      `json:"penName"`
	Lact          int32                       `json:"lact"`
	SameTimeId    int64                       `json:"sameTimeId"`
	SameTimeType  pasturePb.SameTimeType_Kind `json:"sameTimeType"`
	PlanDay       int64                       `json:"planDay"`
	EndDay        int64                       `json:"endDay"`
	RealityDay    int64                       `json:"realityDay"`
	Status        pasturePb.IsShow_Kind       `json:"status"`
	DrugsId       int64                       `json:"drugsId"`
	Unit          pasturePb.Unit_Kind         `json:"unit"`
	Usage         int32                       `json:"usage"`
	OperationId   int64                       `json:"operationId"`
	OperationName string                      `json:"operationName"`
	CreatedAt     int64                       `json:"createdAt"`
	UpdatedAt     int64                       `json:"updatedAt"`
}

func (s *EventCowSameTime) TableName() string {
	return "event_cow_same_time"
}

func NewCowSameTimeDetailList(cowList []*Cow, sameTimeId int64, planTime int64, sameTimeType pasturePb.SameTimeType_Kind) []*EventCowSameTime {
	res := make([]*EventCowSameTime, len(cowList))
	for i, cow := range cowList {
		res[i] = &EventCowSameTime{
			CowId:        cow.Id,
			Lact:         cow.Lact,
			PenId:        cow.PenId,
			SameTimeId:   sameTimeId,
			SameTimeType: sameTimeType,
			Status:       pasturePb.IsShow_No,
			PlanDay:      planTime,
			EndDay:       planTime,
		}
	}

	return res
}

type SameTimeHeader struct {
	Id              string `json:"id"`
	CowId           string `json:"cowId"`
	CowTypeName     string `json:"cowTypeName"`
	BreedStatusName string `json:"breedStatusName"`
	PenName         string `json:"penName"`
	Lact            string `json:"lact"`
	CalvingAge      string `json:"calvingAge"`
	AbortionAge     string `json:"abortionAge"`
	DayAge          string `json:"dayAge"`
	Status          string `json:"status"`
}

type SameTimeBody struct {
	Id              int64                      `json:"id"`
	CowId           int64                      `json:"cowId"`
	BreedStatus     pasturePb.BreedStatus_Kind `json:"breedStatus"`
	BreedStatusName string                     `json:"breedStatusName"`
	CowType         pasturePb.CowType_Kind     `json:"cowType"`
	CowTypeName     string                     `json:"cowTypeName"`
	PenId           int32                      `json:"penId"`
	PenName         string                     `json:"penName"`
	Lact            int32                      `json:"lact"`
	CalvingAge      int32                      `json:"calvingAge"`
	AbortionAge     int32                      `json:"abortionAge"`
	DayAge          int32                      `json:"dayAge"`
	Status          pasturePb.IsShow_Kind      `json:"status"`
}

type SameTimeBodySlice []*SameTimeBody

func (s SameTimeBodySlice) ToPB(
	cowTypeMap map[pasturePb.CowType_Kind]string,
	breedStatusMap map[pasturePb.BreedStatus_Kind]string,
	penMap map[int32]*Pen,
) []*SameTimeBody {
	res := make([]*SameTimeBody, len(s))
	for i, v := range s {
		res[i] = &SameTimeBody{
			Id:              v.Id,
			CowId:           v.CowId,
			CowType:         v.CowType,
			CowTypeName:     cowTypeMap[v.CowType],
			BreedStatus:     v.BreedStatus,
			BreedStatusName: breedStatusMap[v.BreedStatus],
			PenName:         penMap[v.PenId].Name,
			PenId:           v.PenId,
			Lact:            v.Lact,
			CalvingAge:      v.CalvingAge,
			AbortionAge:     v.AbortionAge,
			DayAge:          v.DayAge,
			Status:          v.Status,
		}
	}
	return res
}

type EventCowSameTimeSlice []*EventCowSameTime

func (s EventCowSameTimeSlice) ToPB() []*pasturePb.EventSameTime {
	res := make([]*pasturePb.EventSameTime, len(s))
	for i, v := range s {
		res[i] = &pasturePb.EventSameTime{
			CowId:      fmt.Sprintf("%d", v.CowId),
			DrugsId:    int32(v.DrugsId),
			Usage:      v.Usage,
			Lact:       v.Lact,
			SameTimeId: int32(v.SameTimeId),
		}
	}
	return res
}
